"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var agent_events_conracts_1 = require("./agent-events-conracts");
var agent_instance_info_builder_1 = require("./agent-instance-info-builder");
var machine_info_builder_1 = require("./machine-info-builder");
var nodejs_env_info_builder_1 = require("./nodejs-env-info-builder");
var agent_start_info_builder_1 = require("./agent-start-info-builder");
var backend_proxy_1 = require("../http/backend-proxy");
var validation_utils_1 = require("../utils/validation-utils");
var watchdog_1 = require("../watchdog");
var uuid = require("uuid");
var AgentEventsController = /** @class */ (function () {
    function AgentEventsController(agentConfig, agentType, technology, sendsPing, logger, backendProxy) {
        this.agentId = uuid();
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(agentConfig, "agentConfig");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(agentType, "agentType");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(sendsPing, "sendsPing");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(logger, "logger");
        validation_utils_1.ValidationUtils.verifyNotNullOrEmpty(technology, "technology");
        this.agentConfig = agentConfig;
        this.type = agentType;
        this.logger = logger;
        this.technology = technology;
        this.sendsPing = sendsPing;
        this.shutDownRetries = 0;
        this.backendProxy = backendProxy || this.createBackendProxy();
        this.initWatchdog();
    }
    AgentEventsController.prototype.submitAgentStartedEvent = function (packageJsonFile) {
        return __awaiter(this, void 0, void 0, function () {
            var event, result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.startRequestStatus = StartRequestStatus.PENDING;
                        event = this.buildAgentStartEvent(packageJsonFile || {});
                        return [4 /*yield*/, this.submitAgentEventRequest(event)];
                    case 1:
                        result = _a.sent();
                        this.startRequestStatus = result ? StartRequestStatus.SUCCEED : StartRequestStatus.FAILED;
                        if (this.sendsPing && this.startRequestStatus == StartRequestStatus.SUCCEED) {
                            this._watchdog.start();
                        }
                        else {
                            this.logger.info("Agent start event failed to submit - ping events will not be sent");
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    AgentEventsController.prototype.submitAgentEventRequest = function (event) {
        return __awaiter(this, void 0, void 0, function () {
            var request, e_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        request = {
                            agentId: this.agentId,
                            events: [event],
                            appName: this.agentConfig.appName.value,
                            buildSessionId: this.agentConfig.buildSessionId.value
                        };
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this.backendProxy.submitAgentEvent(request)];
                    case 2:
                        _a.sent();
                        this.logger.info("Event with type '" + event.type + "' submitted successfully");
                        return [2 /*return*/, true];
                    case 3:
                        e_1 = _a.sent();
                        this.logger.warn("Failed to submit Event with type '" + event.type + "'. Error " + e_1);
                        return [2 /*return*/, false];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    AgentEventsController.prototype.buildAgentStartEvent = function (packageJsonFile) {
        var tags = this.resolveTags();
        var tools = this.resolveTools();
        var agentInstanceInfoBuilder = new agent_instance_info_builder_1.AgentInstanceInfoBuilder(this.agentConfig, this.agentId, this.type, tags, tools, this.sendsPing, this.technology, this.logger);
        var machineInfoBuilder = new machine_info_builder_1.MachineInfoBuilder();
        var nodejsEnvInfoBuilder = new nodejs_env_info_builder_1.NodejsEnvInfoBuilder(packageJsonFile.dependencies);
        var agentStartInfoBuilder = new agent_start_info_builder_1.AgentStartInfoBuilder(agentInstanceInfoBuilder, machineInfoBuilder, nodejsEnvInfoBuilder);
        var agentStartInfo = agentStartInfoBuilder.build();
        return this.buildEvent(agent_events_conracts_1.AgentEventCode.AGENT_STARTED, agentStartInfo);
    };
    AgentEventsController.prototype.buildAgentShutdownEvent = function () {
        return this.buildEvent(agent_events_conracts_1.AgentEventCode.AGENT_SHUTDOWN);
    };
    AgentEventsController.prototype.submitAgentShutdownEvent = function () {
        return __awaiter(this, void 0, void 0, function () {
            var event;
            var _this = this;
            return __generator(this, function (_a) {
                this.shutDownRetries++;
                if (this.startRequestStatus == StartRequestStatus.FAILED) {
                    this.logger.debug('Agent start not submitted - not sending shut down events');
                    return [2 /*return*/];
                }
                if (this.shutDownRetries > AgentEventsController.MAX_SHUTDOWN_RETRIES) {
                    this.logger.debug('Stop trying to send shutdown event after 60 seconds ');
                    return [2 /*return*/];
                }
                this._watchdog.stop();
                event = this.buildAgentShutdownEvent();
                if (this.startRequestStatus == StartRequestStatus.PENDING) {
                    setTimeout(function () { return _this.submitAgentShutdownEvent.call(_this); }, 1000);
                }
                else {
                    return [2 /*return*/, this.submitAgentEventRequest(event)];
                }
                return [2 /*return*/];
            });
        });
    };
    // Not happens if start event failed to submit
    AgentEventsController.prototype.submitPingEvent = function () {
        var _this = this;
        this.logger.debug("About to send ping event, current time: " + Date.now());
        var pingEvent = this.buildEvent(agent_events_conracts_1.AgentEventCode.AGENT_PING);
        this.submitAgentEventRequest(pingEvent).then(function () {
            return _this.logger.debug("Ping event submitted successfully");
        });
    };
    Object.defineProperty(AgentEventsController.prototype, "watchdog", {
        get: function () {
            return this._watchdog;
        },
        enumerable: true,
        configurable: true
    });
    AgentEventsController.prototype.resolveTags = function () {
        return [];
    };
    AgentEventsController.prototype.resolveTools = function () {
        return [];
    };
    AgentEventsController.prototype.createBackendProxy = function () {
        var httpConfig = {
            token: this.agentConfig.token.value,
            proxy: this.agentConfig.proxy.value,
            server: this.agentConfig.server.value,
            compressRequests: this.agentConfig.gzip.value
        };
        return new backend_proxy_1.BackendProxy(httpConfig, this.logger);
    };
    AgentEventsController.prototype.initWatchdog = function () {
        var watchdogOptions = { interval: AgentEventsController.PING_INTERVAL, name: "AgentEventsWatchdog", autoReset: true, unref: true };
        var timers = { setTimeout: setTimeout, clearTimeout: clearTimeout };
        this._watchdog = new watchdog_1.Watchdog(watchdogOptions, timers);
        this._watchdog.on("alarm", this.submitPingEvent.bind(this));
    };
    AgentEventsController.prototype.buildEvent = function (type, data) {
        return {
            type: type,
            utcTimestamp_ms: new Date().getTime(),
            data: data
        };
    };
    AgentEventsController.PING_INTERVAL = 2 * 60 * 1000; // 2 Minutes.
    AgentEventsController.MAX_SHUTDOWN_RETRIES = 60;
    return AgentEventsController;
}());
exports.AgentEventsController = AgentEventsController;
var StartRequestStatus;
(function (StartRequestStatus) {
    StartRequestStatus["FAILED"] = "failed";
    StartRequestStatus["SUCCEED"] = "succeed";
    StartRequestStatus["PENDING"] = "pending";
})(StartRequestStatus || (StartRequestStatus = {}));
//# sourceMappingURL=agent-events-controller.js.map